# -*- encoding:utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

import sys
from contextlib import contextmanager

import six


@contextmanager
def captured_output(stream_name):
    """Return a context manager used by captured_stdout/stdin/stderr
    that temporarily replaces the sys stream *stream_name* with a StringIO.
    Note: This function and the following ``captured_std*`` are copied
          from CPython's ``test.support`` module."""
    orig_stdout = getattr(sys, stream_name)
    setattr(sys, stream_name, six.StringIO())
    try:
        yield getattr(sys, stream_name)
    finally:
        setattr(sys, stream_name, orig_stdout)


def captured_stdout():
    """Capture the output of sys.stdout:
       with captured_stdout() as stdout:
           print("hello")
       self.assertEqual(stdout.getvalue(), "hello\n")
    """
    return captured_output("stdout")



# -*- encoding:utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

import ast

__author__ = 'Adam Johnson'
__email__ = 'me@adamj.eu'
__version__ = '1.1.0'


class ComprehensionChecker(object):
    """
    Flake8 plugin to help you write better list/set/dict comprehensions.
    """
    name = 'flake8-comprehensions'
    version = __version__

    def __init__(self, tree, *args, **kwargs):
        self.tree = tree

    message_C400 = 'C400 Unnecessary generator - rewrite as a {type} comprehension.'
    message_C401 = 'C401 Unnecessary list comprehension - rewrite as a {type} comprehension.'
    message_C402 = 'C402 Unnecessary list literal - rewrite as a {type} literal.'

    def run(self):
        for node in ast.walk(self.tree):
            if (
                isinstance(node, ast.Call) and
                len(node.args) == 1
            ):
                if (
                    isinstance(node.args[0], ast.GeneratorExp) and
                    node.func.id in ('list', 'set', 'dict')
                ):
                    yield (
                        node.lineno,
                        node.col_offset,
                        self.message_C400.format(type=node.func.id),
                        type(self),
                    )

                elif (
                    isinstance(node.args[0], ast.ListComp) and
                    node.func.id in ('set', 'dict')
                ):

                    yield (
                        node.lineno,
                        node.col_offset,
                        self.message_C401.format(type=node.func.id),
                        type(self),
                    )

                elif (
                    isinstance(node.args[0], ast.List) and
                    node.func.id in ('set', 'dict')
                ):

                    yield (
                        node.lineno,
                        node.col_offset,
                        self.message_C402.format(type=node.func.id),
                        type(self),
                    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import re
import sys

from setuptools import setup


def get_version(filename):
    """
    Return package version as listed in `__version__` in `filename`.
    """
    init_py = open(filename).read()
    return re.search("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


version = get_version('flake8_comprehensions.py')


if sys.argv[-1] == 'publish':
    if os.system("pip freeze | grep twine"):
        print("twine not installed.\nUse `pip install twine`.\nExiting.")
        sys.exit()
    os.system("python setup.py sdist bdist_wheel")
    os.system("twine upload dist/*")
    print("You probably want to also tag the version now:")
    print("  git tag -a %s -m 'version %s'" % (version, version))
    print("  git push --tags")
    sys.exit()

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

setup(
    name='flake8-comprehensions',
    version='1.1.0',
    description='A flake8 plugin to help you write better list/set/dict '
                'comprehensions.',
    long_description=readme + '\n\n' + history,
    author="Adam Johnson",
    author_email='me@adamj.eu',
    url='https://github.com/adamchainz/flake8-comprehensions',
    entry_points={
        'flake8.extension': [
            'C40 = '
            'flake8_comprehensions:ComprehensionChecker',
        ],
    },
    py_modules=['flake8_comprehensions'],
    include_package_data=True,
    install_requires=[
        'flake8',
    ],
    license="ISCL",
    zip_safe=False,
    keywords='flake8, comprehensions, list comprehension, set comprehension, '
             'dict comprehension',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: ISC License (ISCL)',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
    ],
)



# -*- encoding:utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

import os
import shutil
import sys
from pathlib import Path
from tempfile import mkdtemp
from textwrap import dedent

from flake8.main import main as flake8_main

from .utils import captured_stdout

MODULE_DIR = Path(__file__).parent.resolve()
TMP_DIR = None


def setup_module(module):
    global TMP_DIR
    TMP_DIR = Path(mkdtemp())


def teardown_module(module):
    shutil.rmtree(str(TMP_DIR))


def run_flake8(file_contents):
    with open(str(TMP_DIR / "example.py"), 'w') as tempf:
        tempf.write(dedent(file_contents.lstrip('\n')).strip() + '\n')

    orig_dir = os.getcwd()
    os.chdir(str(TMP_DIR))
    orig_args = sys.argv
    try:
        # Can't pass args to flake8 but can set to sys.argv
        sys.argv = [
            'flake8',
            '--jobs', '1',
            '--exit-zero',
            'example.py',
        ]

        # Run it
        with captured_stdout() as stdout:
            flake8_main()
        out = stdout.getvalue().strip()
        lines = out.split('\n')
        if lines[-1] == '':
            lines = lines[:-1]
        return lines
    finally:
        sys.argv = orig_args
        os.chdir(orig_dir)


def test_C400_list_pass_1():
    errors = run_flake8("""
        foo = [x for x in range(10)]
    """)
    assert errors == []


def test_C400_list_fail_1():
    errors = run_flake8("""
        foo = list(x for x in range(10))
    """)
    assert errors == [
        'example.py:1:7: C400 Unnecessary generator - rewrite as a list comprehension.',
    ]


def test_C400_list_fail_2():
    errors = run_flake8("""
        foobar = list(
            str(x)
            for x
            in range(10)
        )
    """)
    assert errors == [
        'example.py:1:10: C400 Unnecessary generator - rewrite as a list comprehension.',
    ]


def test_C400_set_pass_1():
    errors = run_flake8("""
        foo = {x for x in range(10)}
    """)
    assert errors == []


def test_C400_set_fail_1():
    errors = run_flake8("""
        foo = set(x for x in range(10))
    """)
    assert errors == [
        'example.py:1:7: C400 Unnecessary generator - rewrite as a set comprehension.',
    ]


def test_C400_set_fail_2():
    errors = run_flake8("""
        foobar = set(
            str(x) for x
            in range(10)
        )
    """)
    assert errors == [
        'example.py:1:10: C400 Unnecessary generator - rewrite as a set comprehension.',
    ]


def test_C400_dict_pass_1():
    errors = run_flake8("""
        foo = {x: str(x) for x in range(10)}
    """)
    assert errors == []


def test_C400_dict__fail_1():
    errors = run_flake8("""
        foo = dict((x, str(x)) for x in range(10))
    """)
    assert errors == [
        'example.py:1:7: C400 Unnecessary generator - rewrite as a dict comprehension.',
    ]


def test_C400_dict__fail_2():
    errors = run_flake8("""
        foobar = dict(
            (x, str(x))
            for x
            in range(10)
        )
    """)
    assert errors == [
        'example.py:1:10: C400 Unnecessary generator - rewrite as a dict comprehension.',
    ]


def test_C401_set_pass_1():
    errors = run_flake8("""
        foo = {x for x in range(10)}
    """)
    assert errors == []


def test_C401_set_fail_1():
    errors = run_flake8("""
        foo = set([x for x in range(10)])
    """)
    assert errors == [
        'example.py:1:7: C401 Unnecessary list comprehension - rewrite as a set comprehension.',
    ]


def test_C401_dict_pass_1():
    errors = run_flake8("""
        foo = {x: x for x in range(10)}
    """)
    assert errors == []


def test_C401_dict_fail_1():
    errors = run_flake8("""
        foo = dict([(x, x) for x in range(10)])
    """)
    assert errors == [
        'example.py:1:7: C401 Unnecessary list comprehension - rewrite as a dict comprehension.',
    ]


def test_C402_set_pass_1():
    errors = run_flake8("""
        foo = set(range)
    """)
    assert errors == []


def test_C402_set_fail_1():
    errors = run_flake8("""
        foo = set([])
    """)
    assert errors == [
        'example.py:1:7: C402 Unnecessary list literal - rewrite as a set literal.',
    ]


def test_C402_set_fail_2():
    errors = run_flake8("""
        foo = set([1])
    """)
    assert errors == [
        'example.py:1:7: C402 Unnecessary list literal - rewrite as a set literal.',
    ]


def test_C402_dict_pass_1():
    errors = run_flake8("""
        foo = dict(range)
    """)
    assert errors == []


def test_C402_dict_fail_1():
    errors = run_flake8("""
        foo = dict([])
    """)
    assert errors == [
        'example.py:1:7: C402 Unnecessary list literal - rewrite as a dict literal.',
    ]


def test_C402_dict_fail_2():
    errors = run_flake8("""
        foo = dict([(1, 2)])
    """)
    assert errors == [
        'example.py:1:7: C402 Unnecessary list literal - rewrite as a dict literal.',
    ]
